@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 # Nagstamon - Nagios status monitor for your desktop
-# Copyright (C) 2008-2019 Henri Wahl <h.wahl@ifw-dresden.de> et al.
+# Copyright (C) 2008-2020 Henri Wahl <h.wahl@ifw-dresden.de> et al.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ import configparser
 import base64
 import zlib
 import datetime
+from urllib.parse import quote
 from collections import OrderedDict
 
 # older Kubuntu has trouble with keyring
@@ -40,6 +41,33 @@ if platform.system() == 'Linux':
 if KEYRING:
     import keyring
 
+# instead of calling platform.system() every now and then just do it once here
+OS = platform.system()
+# needed when OS-specific decisions have to be made, mostly Linux/non-Linux
+OS_DARWIN = 'Darwin'
+OS_WINDOWS = 'Windows'
+OS_NON_LINUX = (OS_DARWIN, OS_WINDOWS)
+
+# simple Wayland detection
+if 'WAYLAND_DISPLAY' in os.environ or\
+    'XDG_SESSION_TYPE' in os.environ and os.environ['XDG_SESSION_TYPE'] == 'wayland':
+    DESKTOP_WAYLAND = True
+else:
+    DESKTOP_WAYLAND = False
+
+# Cinnamon detection
+# CINNAMON_VERSION 4.2.4
+# DESKTOP_SESSION cinnamon
+# GDMSESSION cinnamon
+# XDG_CURRENT_DESKTOP X-Cinnamon
+# XDG_SESSION_DESKTOP cinnamon
+if 'CINNAMON_VERSION' in os.environ or\
+    'DESKTOP_SESSION' in os.environ and os.environ['DESKTOP_SESSION'] == 'cinnamon' or \
+    'XDG_SESSION_DESKTOP' in os.environ and os.environ['XDG_SESSION_DESKTOP'] == 'cinnamon':
+    DESKTOP_CINNAMON = True
+else:
+    DESKTOP_CINNAMON = False
+
 # queue.Queue() needs threading module which might be not such a good idea to be used
 # because QThread is already in use
 # maybe not the most logical place here to be defined but at least all
@@ -68,6 +96,9 @@ CONFIG_STRINGS = ['custom_browser',
                   're_host_pattern',
                   're_service_pattern',
                   're_status_information_pattern',
+                  're_duration_pattern',
+                  're_attempt_pattern',
+                  're_groups_pattern',
                   're_criticality_pattern',
                   'font',
                   'defaults_acknowledge_comment',
@@ -86,34 +117,21 @@ CONFIG_STRINGS = ['custom_browser',
                   'monitor_site'
                   ]
 
-# needed when OS-specific decisions have to be made, mostly Linux/non-Linux
-NON_LINUX = ('Darwin', 'Windows')
-
-# invalid characters in path names of config files have to be replaced
-INVALID_CHARACTERS = {'\\': '_backslash_',
-                      '/': '_slash_',
-                      ':': '_colon_',
-                      '*': '_aterisk_',
-                      '?': '_question_mark_',
-                      '"': '_double_quotes_',
-                      '<': '_less_than_',
-                      '>': '_greater_than_'}
-
 
 class AppInfo(object):
     """
         contains app information previously located in GUI.py
     """
     NAME = 'Nagstamon'
-    VERSION = '3.2.1'
+    VERSION = '3.4.1'
     WEBSITE = 'https://nagstamon.ifw-dresden.de'
-    COPYRIGHT = '©2008-2019 Henri Wahl et al.'
+    COPYRIGHT = '©2008-2020 Henri Wahl et al.'
     COMMENTS = 'Nagios status monitor for your desktop'
     # version URL depends on version string
     if 'alpha' in VERSION.lower() or \
-            'beta' in VERSION.lower() or \
-            'rc' in VERSION.lower() or \
-            '-' in VERSION.lower():
+        'beta' in VERSION.lower() or \
+        'rc' in VERSION.lower() or \
+        '-' in VERSION.lower():
         VERSION_URL = WEBSITE + '/version/unstable'
     else:
         VERSION_URL = WEBSITE + '/version/stable'
@@ -223,6 +241,15 @@ class Config(object):
         self.re_status_information_enabled = False
         self.re_status_information_pattern = ''
         self.re_status_information_reverse = False
+        self.re_duration_enabled = False
+        self.re_duration_pattern = ''
+        self.re_duration_reverse = False
+        self.re_attempt_enabled = False
+        self.re_attempt_pattern = ''
+        self.re_attempt_reverse = False
+        self.re_groups_enabled = False
+        self.re_groups_pattern = ''
+        self.re_groups_reverse = False
         self.color_ok_text = self.default_color_ok_text = '#FFFFFF'
         self.color_ok_background = self.default_color_ok_background = '#006400'
         self.color_information_text = self.default_color_information_text = "#000000"
@@ -257,6 +284,9 @@ class Config(object):
         self.defaults_acknowledge_send_notification = False
         self.defaults_acknowledge_persistent_comment = False
         self.defaults_acknowledge_all_services = False
+        self.defaults_acknowledge_expire = False
+        self.defaults_acknowledge_expire_duration_hours = 2
+        self.defaults_acknowledge_expire_duration_minutes = 0
         self.defaults_acknowledge_comment = 'acknowledged'
         self.defaults_submit_check_result_comment = 'check result submitted'
         self.defaults_downtime_duration_hours = 2
@@ -373,7 +403,7 @@ class Config(object):
                     self.use_system_keyring = False
                 else:
                     # a configured system seemed to be able to run and thus use system keyring
-                    if platform.system() in NON_LINUX:
+                    if OS in OS_NON_LINUX:
                         self.use_system_keyring = True
                     else:
                         self.use_system_keyring = self.KeyringAvailable()
@@ -448,6 +478,16 @@ class Config(object):
                 if servers[server].type == 'Icinga2':
                     servers[server].type = 'IcingaWeb2'
 
+                # Check_MK is now Checkmk - same renaming as with IcingaWeb2
+                if servers[server].type == 'Check_MK Multisite':
+                    servers[server].type = 'Checkmk Multisite'
+                if 'check_mk_view_hosts' in servers[server].__dict__.keys():
+                    servers[server].checkmk_view_hosts = servers[server].check_mk_view_hosts
+                    servers[server].__dict__.pop('check_mk_view_hosts')
+                if 'check_mk_view_services' in servers[server].__dict__.keys():
+                    servers[server].checkmk_view_services = servers[server].check_mk_view_services
+                    servers[server].__dict__.pop('check_mk_view_services')
+
         except Exception:
             import traceback
             traceback.print_exc(file=sys.stdout)
@@ -463,10 +503,10 @@ class Config(object):
 
         try:
             if os.path.exists(self.configdir + os.sep + settingsdir):
-                for f in sorted(os.listdir(self.configdir + os.sep + settingsdir)):
-                    if f.startswith(setting + '_') and f.endswith('.conf'):
+                for settingsfile in sorted(os.listdir(self.configdir + os.sep + settingsdir)):
+                    if settingsfile.startswith(setting + '_') and settingsfile.endswith('.conf'):
                         config = configparser.ConfigParser(allow_no_value=True, interpolation=None)
-                        config.read(self.configdir + os.sep + settingsdir + os.sep + f)
+                        config.read(self.configdir + os.sep + settingsdir + os.sep + settingsfile)
 
                         # create object for every setting
                         name = config.sections()[0].replace(setting + '_', '', 1)
@@ -486,6 +526,13 @@ class Config(object):
                             else:
                                 value = i[1]
                             settings[name].__setattr__(i[0], value)
+
+                        # if filename is still one of the non-URL-ones delete duplicate file
+                        if settingsfile != '{0}_{1}.conf'.format(setting, quote(name, safe='')):
+                            self.delete_file(settingsdir, settingsfile)
+                            # set flag to store the settings via legacy adjustments
+                            self.save_config_after_urlencode = True
+
         except Exception:
             import traceback
             traceback.print_exc(file=sys.stdout)
@@ -518,21 +565,20 @@ class Config(object):
                     self.use_system_keyring = False
                 else:
                     # a configured system seemed to be able to run and thus use system keyring
-                    if platform.system() in NON_LINUX:
+                    if OS in OS_NON_LINUX:
                         self.use_system_keyring = True
                     else:
                         self.use_system_keyring = self.KeyringAvailable()
 
-            # save actions dict
+            # save servers dict
             self.SaveMultipleConfig('servers', 'server')
 
             # save actions dict
             self.SaveMultipleConfig('actions', 'action')
 
             # open, save and close config file
-            f = open(os.path.normpath(self.configfile), "w")
-            config.write(f)
-            f.close()
+            with open(os.path.normpath(self.configfile), 'w') as file:
+                config.write(file)
 
             # debug
             if self.debug_mode:
@@ -614,16 +660,15 @@ class Config(object):
             if not os.path.exists(self.configdir + os.sep + settingsdir):
                 os.makedirs(self.configdir + os.sep + settingsdir)
 
-            # replace invalid characters by their literal replacements
-            s_clean = s
-            for c in INVALID_CHARACTERS:
-                s_clean = s_clean.replace(c, INVALID_CHARACTERS[c])
-
-            f = open(
-                os.path.normpath(self.configdir + os.sep + settingsdir + os.sep + setting + "_" + s_clean + ".conf"),
-                "w")
-            config.write(f)
-            f.close()
+            # quote strings and thus avoid invalid characters
+            with open(
+                    os.path.normpath('{0}{1}{2}{1}{3}_{4}.conf'.format(self.configdir,
+                                                                       os.sep,
+                                                                       settingsdir,
+                                                                       setting,
+                                                                       quote(s, safe=''))),
+                    'w') as file:
+                config.write(file)
 
             # ### clean up old deleted/renamed config files
             # ##if os.path.exists(self.configdir + os.sep + settingsdir):
@@ -638,7 +683,7 @@ class Config(object):
         try:
             # Linux systems should use keyring only if it comes with the distro, otherwise chances are small
             # that keyring works at all
-            if platform.system() in NON_LINUX:
+            if OS in OS_NON_LINUX:
                 # safety first - if not yet available disable it
                 if 'use_system_keyring' not in self.__dict__.keys():
                     self.use_system_keyring = False
@@ -711,7 +756,7 @@ class Config(object):
         """
         create some default actions like SSH and so on
         """
-        if platform.system() == "Windows":
+        if OS == OS_WINDOWS:
             defaultactions = {"RDP": Action(name="RDP", description="Connect via RDP.",
                                             type="command", string="C:\windows\system32\mstsc.exe /v:$ADDRESS$"),
                               "VNC": Action(name="VNC", description="Connect via VNC.",
@@ -721,7 +766,7 @@ class Config(object):
                               "SSH": Action(name="SSH", description="Connect via SSH.",
                                             type="command",
                                             string="C:\Program Files\PuTTY\putty.exe -l root $ADDRESS$")}
-        elif platform.system() == "Darwin":
+        elif OS == OS_DARWIN:
             defaultactions = {"RDP": Action(name="RDP", description="Connect via RDP.",
                                             type="command", string="open rdp://$ADDRESS$"),
                               "VNC": Action(name="VNC", description="Connect via VNC.",
@@ -769,18 +814,18 @@ class Config(object):
                                                            filter_target_host=True,
                                                            string="$MONITOR$/event?host=$HOST$&service=$SERVICE$",
                                                            enabled=False)
-        defaultactions["Check_MK-1-Click-Acknowledge-Host"] = Action(name="Check_MK-1-Click-Acknowledge-Host",
+        defaultactions["Checkmk-1-Click-Acknowledge-Host"] = Action(name="Checkmk-1-Click-Acknowledge-Host",
                                                                      type="url",
                                                                      description="Acknowledges a host with one click.",
                                                                      filter_target_service=False, enabled=False,
                                                                      string="$MONITOR$/view.py?_transid=$TRANSID$&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=hoststatus&host=$HOST$&_ack_comment=$COMMENT-ACK$&_acknowledge=Acknowledge")
-        defaultactions["Check_MK-1-Click-Acknowledge-Service"] = Action(name="Check_MK-1-Click-Acknowledge-Service",
+        defaultactions["Checkmk-1-Click-Acknowledge-Service"] = Action(name="Checkmk-1-Click-Acknowledge-Service",
                                                                         type="url",
                                                                         description="Acknowledges a host with one click.",
                                                                         filter_target_host=False, enabled=False,
                                                                         string="$MONITOR$/view.py?_transid=$TRANSID$&_do_actions=yes&_do_confirm=Yes!&output_format=python&view_name=service&host=$HOST$&_ack_comment=$COMMENT-ACK$&_acknowledge=Acknowledge&service=$SERVICE$")
-        defaultactions["Check_MK Edit host in WATO"] = Action(name="Check_MK Edit host in WATO", enabled=False,
-                                                              monitor_type="Check_MK Multisite",
+        defaultactions["Checkmk Edit host in WATO"] = Action(name="Checkmk Edit host in WATO", enabled=False,
+                                                              monitor_type="Checkmk Multisite",
                                                               description="Edit host in WATO.",
                                                               string="$MONITOR$index.py?start_url=%2Fmonitor%2Fcheck_mk%2Fwato.py%3Fhost%3D$HOST$%26mode%3Dedit_host")
         defaultactions["Email"] = Action(name="Email", enabled=False, description="Send email to someone.",
@@ -791,15 +836,15 @@ class Config(object):
 
     def _LegacyAdjustments(self):
         # mere cosmetics but might be more clear for future additions - changing any "nagios"-setting to "monitor"
-        for s in self.servers.values():
-            if 'nagios_url' in s.__dict__.keys():
-                s.monitor_url = s.nagios_url
-            if 'nagios_cgi_url' in s.__dict__.keys():
-                s.monitor_cgi_url = s.nagios_cgi_url
+        for server in self.servers.values():
+            if 'nagios_url' in server.__dict__.keys():
+                server.monitor_url = server.nagios_url
+            if 'nagios_cgi_url' in server.__dict__.keys():
+                server.monitor_cgi_url = server.nagios_cgi_url
 
             # to reduce complexity in Centreon there is also only one URL necessary
-            if s.type == "Centreon":
-                s.monitor_url = s.monitor_cgi_url
+            if server.type == "Centreon":
+                server.monitor_url = server.monitor_cgi_url
 
         # switch to update interval in seconds not minutes
         if 'update_interval' in self.__dict__.keys():
@@ -818,6 +863,10 @@ class Config(object):
                 # set browser as default to make user notice something is wrong
                 action.type = 'browser'
 
+        # might be there only once after starting Nagstamon 3.4 the first time
+        if 'save_config_after_urlencode' in self.__dict__.keys():
+            self.SaveConfig()
+
     def GetNumberOfEnabledMonitors(self):
         """
             returns the number of enabled monitors - in case all are disabled there is no need to display the popwin
@@ -835,7 +884,7 @@ class Config(object):
             delete specified .conf file if setting is deleted in GUI
         """
         # clean up old deleted/renamed config file
-        file = os.path.abspath('{1}{0}{2}{0}{3}.conf'.format(os.sep, self.configdir, settings_dir, settings_file))
+        file = os.path.abspath('{1}{0}{2}{0}{3}'.format(os.sep, self.configdir, settings_dir, settings_file))
         if os.path.exists(file) and (os.path.isfile(file) or os.path.islink(file)):
             try:
                 os.unlink(file)
@@ -886,12 +935,12 @@ class Server(object):
         # IcingaWeb2 might authenticate without cookies too - default is WITH cookies
         self.no_cookie_auth = False
 
-        # Check_MK Multisite
-        # Force Check_MK livestatus code to set AuthUser header for users who
+        # Checkmk Multisite
+        # Force Checkmk livestatus code to set AuthUser header for users who
         # are permitted to see all objects.
         self.force_authuser = False
-        self.check_mk_view_hosts = 'nagstamon_hosts'
-        self.check_mk_view_services = 'nagstamon_svc'
+        self.checkmk_view_hosts = 'nagstamon_hosts'
+        self.checkmk_view_services = 'nagstamon_svc'
 
         # OP5 api filters
         self.host_filter = 'state !=0'
@@ -939,6 +988,15 @@ class Action(object):
         self.re_status_information_enabled = False
         self.re_status_information_pattern = ""
         self.re_status_information_reverse = False
+        self.re_duration_enabled = False
+        self.re_duration_pattern = ""
+        self.re_duration_reverse = False
+        self.re_attempt_enabled = False
+        self.re_attempt_pattern = ""
+        self.re_attempt_reverse = False
+        self.re_groups_enabled = False
+        self.re_groups_pattern = ""
+        self.re_groups_reverse = False
         # close powin or not, depends on personal preference
         self.close_popwin = True
         self.leave_popwin_open = False
