@@ -63,6 +63,8 @@ class CentreonServer(GenericServer):
     first_login = True
     # limit number of services retrived
     limit_services_number = 9999
+    # default value, applies to version 2.2 and others
+    XML_PATH = 'xml'
 
     def init_config(self):
         '''
@@ -70,7 +72,7 @@ class CentreonServer(GenericServer):
         '''
         # set URLs here already
         self.init_HTTP()
-        if not self.tls_error:
+        if not self.tls_error and self.centreon_version is not None:
             self._define_url()
 
     def init_HTTP(self):
@@ -96,7 +98,7 @@ class CentreonServer(GenericServer):
                 elif re.search('2\.[3-6]\.[0-5]', raw_versioncheck):
                     self.centreon_version = 2.3456
                     if conf.debug_mode is True:
-                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.6.5 <=> 2.3')
+                        self.Debug(server=self.get_name(), debug='Centreon version detected : 2.3 <=> 2.6.5')
                     # URLs for browser shortlinks/buttons on popup window
                     self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?p=1',
                                         'hosts': '$MONITOR$/main.php?p=20103&o=hpb',
@@ -131,10 +133,10 @@ class CentreonServer(GenericServer):
                                         'hosts': '$MONITOR$/main.php?p=20202',
                                         'services': '$MONITOR$/main.php?p=20201',
                                         'history': '$MONITOR$/main.php?p=203'}
-                elif re.search('18\.10\.[0-9]', raw_versioncheck):
+                elif re.search('18\.10\.[0-9]', raw_versioncheck) or re.search('19\.(04|10)\.[0-9]', raw_versioncheck):
                     self.centreon_version = 18.10
                     if conf.debug_mode is True:
-                        self.Debug(server=self.get_name(), debug='Centreon version detected : 18.10')
+                        self.Debug(server=self.get_name(), debug='Centreon version detected : 18.10 <=> 19.10')
                     # URLs for browser shortlinks/buttons on popup window
                     self.BROWSER_URLS = {'monitor': '$MONITOR$/main.php?',
                                         'hosts': '$MONITOR$/main.php?p=20202',
@@ -229,6 +231,11 @@ class CentreonServer(GenericServer):
                 raw = self.FetchURL(self.monitor_cgi_url + '/index.php?p=101&autologin=1&useralias=' + self.username + '&token=' + self.autologin_key, giveback='raw')
                 if conf.debug_mode == True:
                     self.Debug(server=self.get_name(), debug='Autologin : ' + self.username + ' : ' + self.autologin_key)
+                # Gathering of the token who will be used to interact with Centreon
+                page = self.FetchURL(self.monitor_cgi_url + '/main.get.php')
+                if  self.centreon_version in [18.10, 2.8, 2.7, 2.66]:
+                    self.centreon_token = page.result.find('input', {'name': "centreon_token"})['value']
+
             # Password auth
             else:
                 login = self.FetchURL(self.monitor_cgi_url + '/index.php')
@@ -250,9 +257,11 @@ class CentreonServer(GenericServer):
                         raw = self.FetchURL(self.monitor_cgi_url + "/index.php",cgi_data=login_data, giveback="raw")
                 if conf.debug_mode == True:
                     self.Debug(server=self.get_name(), debug='Password login : ' + self.username + ' : ' + self.password)
+
             sid = self.session.cookies['PHPSESSID']
             if conf.debug_mode == True:
                 self.Debug(server=self.get_name(), debug='SID : ' + sid)
+                self.Debug(server=self.get_name(), debug='Centreon Token : ' + self.centreon_token)
             # those broker urls would not be changing too often so this check migth be done here
             if self.first_login:
                 self._get_xml_path(sid)
@@ -376,9 +385,7 @@ class CentreonServer(GenericServer):
         Centreon 2.8 + c.broker - /include/monitoring/status/Hosts/xml/hostXML.php
         regexping HTML for Javascript
         '''
-        if self.centreon_version == 2.2:
-            self.XML_PATH = 'xml'
-        elif self.centreon_version == 2.3456 or self.centreon_version == 2.66:
+        if self.centreon_version == 2.3456 or self.centreon_version == 2.66:
             # 2.6 support NDO and C. Broker, we must check which one is used
             # cgi_data = {'p':201, 'sid':self.SID}
             cgi_data = {'p':201, 'sid':sid}
@@ -403,7 +410,6 @@ class CentreonServer(GenericServer):
                     self.Debug(server=self.get_name(), debug='Unable to fetch the main page to detect the broker : ' + error)
             del result, error
         else:
-            self.XML_PATH = 'xml'
             if conf.debug_mode == True:
                 self.Debug(server=self.get_name(), debug='Only Centreon Broker is supported in Centeon >= 2.7 -> XML_PATH='+ self.XML_PATH)
 
@@ -458,7 +464,6 @@ class CentreonServer(GenericServer):
             'external_cmd_cmdPopup': self.monitor_cgi_url + '/include/monitoring/external_cmd/cmdPopup.php',
             'keepAlive': self.monitor_cgi_url + '/api/internal.php?object=centreon_keepalive&action=keepAlive'
         }
-
         if self.centreon_version < 2.7:
             self.urls_centreon = urls_centreon_2_2
         elif self.centreon_version == 2.7:
@@ -545,13 +550,17 @@ class CentreonServer(GenericServer):
         except:
             return '',''
 
-
     def _get_status(self):
         '''
         Get status from Centreon Server
         '''
         # Be sure that the session is still active
-        self._check_session()
+        result = self._check_session()
+        if result is not None:
+            if result.result == 'ERROR':
+                if 'urls_centreon' in result.error:
+                    result.error = 'Connection error'
+                return result
 
         # services (unknown, warning or critical?)
         if self.centreon_version in [2.7, 2.8, 18.10]:
@@ -1015,12 +1024,16 @@ class CentreonServer(GenericServer):
     def _check_session(self):
         if conf.debug_mode == True:
             self.Debug(server=self.get_name(), debug='Checking session status')
+        if 'url_centreon' not in self.__dict__:
+            self.init_config()
         try:
             if self.centreon_version == 18.10:
                 result = self.FetchURL(self.urls_centreon['keepAlive'], giveback='raw')
                 raw, error, status_code = result.result, result.error, result.status_code
+                # Return 200 & null a session is open
                 if conf.debug_mode == True:
                     self.Debug(server=self.get_name(), debug='Session status : ' + raw)
+                # 401 if no valid session is present
                 if self.status_code == 401:
                     self.SID = self._get_sid().result
                     if conf.debug_mode == True:
