@@ -1,7 +1,7 @@
 # encoding: utf-8
 
 # Nagstamon - Nagios status monitor for your desktop
-# Copyright (C) 2008-2019 Henri Wahl <h.wahl@ifw-dresden.de> et al.
+# Copyright (C) 2008-2020 Henri Wahl <h.wahl@ifw-dresden.de> et al.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,18 +17,25 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-import sys
-import socket
+from collections import OrderedDict
 import copy
 import datetime
-import traceback
+from pathlib import Path
 import platform
+import socket
+import sys
+import traceback
 import urllib.parse
+
 from bs4 import BeautifulSoup
+import requests
 
 from Nagstamon.Helpers import (host_is_filtered_out_by_re,
                                ServiceIsFilteredOutByRE,
                                StatusInformationIsFilteredOutByRE,
+                               DurationIsFilteredOutByRE,
+                               AttemptIsFilteredOutByRE,
+                               GroupsIsFilteredOutByRE,
                                CriticalityIsFilteredOutByRE,
                                not_empty,
                                webbrowser_open,
@@ -38,14 +45,22 @@ from Nagstamon.Objects import (GenericService,
                                GenericHost,
                                Result)
 
-from Nagstamon.Config import (conf,
-                              AppInfo,
-                              debug_queue)
+from Nagstamon.Config import (AppInfo,
+                              conf,
+                              debug_queue,
+                              OS,
+                              OS_DARWIN,
+                              RESOURCES)
 
-from collections import OrderedDict
 
-import requests
-import requests_kerberos
+# requests_gssapi is newer but not available everywhere
+try:
+    # extra imports needed to get it compiled on macOS
+    import numbers
+    import gssapi.raw.cython_converters
+    from requests_gssapi import HTTPSPNEGOAuth as HTTPSKerberos
+except ImportError:
+    from requests_kerberos import HTTPKerberosAuth as HTTPSKerberos
 
 # disable annoying SubjectAltNameWarning warnings
 try:
@@ -114,12 +129,13 @@ class GenericServer(object):
         self.proxy_username = ''
         self.proxy_password = ''
         self.auth_type = ''
+        self.encoding = None
         self.hosts = dict()
         self.new_hosts = dict()
         self.isChecking = False
         self.CheckingForNewVersion = False
-        # store current and difference of worst state for notification
-        self.worst_status_diff = self.worst_status_current = 'UP'
+        # store current, last and difference of worst state for notification
+        self.worst_status_diff = self.worst_status_current = self.worst_status_last = 'UP'
         self.nagitems_filtered_list = list()
         self.nagitems_filtered = {'services': {'DISASTER': [], 'CRITICAL': [], 'HIGH': [],
             'AVERAGE': [], 'WARNING': [], 'INFORMATION': [], 'UNKNOWN': []},
@@ -169,6 +185,19 @@ class GenericServer(object):
         # to handle Icinga versions this information is necessary, might be of future use for others too
         self.version = ''
 
+        # macOS pyinstaller onefile conglomerate tends to lose cacert.pem due to macOS temp folder cleaning
+        self.cacert_path = self.cacert_content = False
+        if OS == OS_DARWIN:
+            # trying to find root path when run by pyinstaller onefile, must be something like
+            # /var/folders/7w/hfvrg7v92x3gjt95cqh974240000gn/T/_MEIQ3l3u3
+            root_path = Path(RESOURCES).parent.parent
+            if root_path.joinpath('certifi').is_dir() and root_path.joinpath('certifi', 'cacert.pem').is_file():
+                # store path of cacert...
+                self.cacert_path = root_path.joinpath('certifi', 'cacert.pem')
+                # ...and its content
+                with open(self.cacert_path, mode='rb') as file:
+                    self.cacert_content = file.read()
+
         # Special FX
         # Centreon
         self.use_autologin = False
@@ -176,7 +205,7 @@ class GenericServer(object):
         # Icinga
         self.use_display_name_host = False
         self.use_display_name_service = False
-        # Check_MK Multisite
+        # Checkmk Multisite
         self.force_authuser = False
 
         # OP5 api filters
@@ -232,7 +261,7 @@ class GenericServer(object):
             elif self.authentication == 'digest':
                 self.session.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
             elif self.authentication == 'kerberos':
-                self.session.auth = requests_kerberos.HTTPKerberosAuth()
+                self.session.auth = HTTPSKerberos()
 
             # default to check TLS validity
             if self.ignore_cert:
@@ -845,8 +874,8 @@ class GenericServer(object):
                           status_code=self.status_code)
 
         if (self.status == 'ERROR' or
-         self.status_description != '' or
-         self.status_code >= 400):
+            self.status_description != '' or
+            self.status_code >= 400):
 
             # ask for password if authorization failed
             if 'HTTP Error 401' in self.status_description or \
@@ -928,7 +957,7 @@ class GenericServer(object):
                         self.Debug(server=self.get_name(), debug='Filter: FLAPPING HOST ' + str(host.name))
                     host.visible = False
 
-                # Check_MK and OP5 do not show the status_type so their host.status_type will be empty
+                # Checkmk and OP5 do not show the status_type so their host.status_type will be empty
                 if host.status_type != '':
                     if conf.filter_hosts_in_soft_state is True and host.status_type == 'soft':
                         if conf.debug_mode:
@@ -1046,7 +1075,7 @@ class GenericServer(object):
                                        service.name))
                     service.visible = False
 
-                # Check_MK and OP5 do not show the status_type so their host.status_type will be empty
+                # Checkmk and OP5 do not show the status_type so their host.status_type will be empty
                 if service.status_type != '':
                     if conf.filter_services_in_soft_state is True and service.status_type == 'soft':
                         if conf.debug_mode:
@@ -1087,6 +1116,24 @@ class GenericServer(object):
                                    debug='Filter: REGEXP ' + str(host.name) + ';' + str(service.name))
                     service.visible = False
 
+                if DurationIsFilteredOutByRE(service.duration, conf) is True:
+                    if conf.debug_mode:
+                        self.Debug(server=self.get_name(),
+                                   debug='Filter: REGEXP ' + str(host.name) + ';' + str(service.name))
+                    service.visible = False
+
+                if AttemptIsFilteredOutByRE(service.attempt, conf) is True:
+                    if conf.debug_mode:
+                        self.Debug(server=self.get_name(),
+                                   debug='Filter: REGEXP ' + str(host.name) + ';' + str(service.name))
+                    service.visible = False
+
+                if GroupsIsFilteredOutByRE(service.groups, conf) is True:
+                    if conf.debug_mode:
+                        self.Debug(server=self.get_name(),
+                                   debug='Filter: REGEXP ' + str(host.name) + ';' + str(service.name))
+                    service.visible = False
+
                 # The Criticality filter can be used only with centreon objects. Other objects don't have the criticality attribute.
                 if self.type == 'Centreon':
                     if CriticalityIsFilteredOutByRE(service.criticality, conf):
@@ -1273,8 +1320,10 @@ class GenericServer(object):
 
                 # final worst state is one of the predefined states
                 self.worst_status_diff = STATES[worst]
+                del diff_states
 
         # get the current worst state, needed at least for systraystatusicon
+        self.worst_status_last = self.worst_status_current
         self.worst_status_current = 'UP'
         if self.down > 0:
             self.worst_status_current = 'DOWN'
@@ -1366,11 +1415,41 @@ class GenericServer(object):
             try:
                 # debug
                 if conf.debug_mode is True:
-                    self.Debug(server=self.get_name(), debug='FetchURL: ' + url + ' CGI Data: ' + str(cgi_data))
+                    # unpasswordify CGI data
+                    if cgi_data is not None and not isinstance(cgi_data, str):
+                        cgi_data_log = copy.copy(cgi_data)
+                        for key in cgi_data_log.keys():
+                            if 'pass' in key:
+                                cgi_data_log[key] = '***************'
+                    else:
+                        cgi_data_log = cgi_data
+                    self.Debug(server=self.get_name(), debug='FetchURL: ' + url + ' CGI Data: ' + str(cgi_data_log))
+
+                if OS == OS_DARWIN and not self.cacert_path.is_file():
+                    # pyinstaller temp folder seems to be emptied completely after a while
+                    # so the directories containing the resources have to be recreated too
+                    self.cacert_path.parent.mkdir(exist_ok=True)
+                    # write cached content of cacert.pem file back onto disk
+                    with self.cacert_path.open(mode='wb') as file:
+                        file.write(self.cacert_content)
+
+                # in case we know the server's encoding use it
+                if self.encoding:
+                    if cgi_data is not None:
+                        try:
+                            for k in cgi_data:
+                                cgi_data[k] = cgi_data[k].encode(self.encoding)
+                        except:
+                            # set to false to mark it as invalid
+                            self.encoding = False
 
                 # use session only for connections to monitor servers, other requests like looking for updates
                 # should go out without credentials
                 if no_auth is False:
+                    # check if there is really a session
+                    if not self.session:
+                        self.reset_HTTP()
+                        self.init_HTTP()
                     # most requests come without multipart/form-data
                     if multipart is False:
                         if cgi_data is None:
@@ -1378,7 +1457,7 @@ class GenericServer(object):
                         else:
                             response = self.session.post(url, data=cgi_data, timeout=self.timeout)
                     else:
-                        # Check_MK and Opsview need multipart/form-data encoding
+                        # Checkmk and Opsview need multipart/form-data encoding
                         # http://stackoverflow.com/questions/23120974/python-requests-post-multipart-form-data-without-filename-in-http-request#23131823
                         form_data = dict()
                         for key in cgi_data:
@@ -1395,7 +1474,7 @@ class GenericServer(object):
                     self.proxify(temporary_session)
 
                     # no need to check TLS validity for temporary sessions like update check
-                    temporary_session.verify = False
+                    ###temporary_session.verify = False
 
                     # most requests come without multipart/form-data
                     if multipart is False:
@@ -1404,7 +1483,7 @@ class GenericServer(object):
                         else:
                             response = temporary_session.post(url, data=cgi_data, timeout=self.timeout)
                     else:
-                        # Check_MK and Opsview nees multipart/form-data encoding
+                        # Checkmk and Opsview need multipart/form-data encoding
                         # http://stackoverflow.com/questions/23120974/python-requests-post-multipart-form-data-without-filename-in-http-request#23131823
                         form_data = dict()
                         for key in cgi_data:
@@ -1424,6 +1503,10 @@ class GenericServer(object):
                     self.tls_error = False
                 return Result(result=result, error=error, status_code=-1)
 
+            # store encoding in case it is not set yet and is not False
+            if self.encoding is None:
+                self.encoding = response.encoding
+
             # give back pure HTML or XML in case giveback is 'raw'
             if giveback == 'raw':
                 # .text gives content in unicode
@@ -1579,3 +1662,36 @@ class GenericServer(object):
                           status_code=copy.deepcopy(status_code)))
         else:
             return(False)
+
+    def get_worst_status_current(self):
+        """
+            hand over the current worst status for get_worst_status()
+        """
+        # get the current worst state, needed at least for systraystatusicon
+        self.worst_status_current = 'UP'
+        if self.down > 0:
+            self.worst_status_current = 'DOWN'
+        elif self.unreachable > 0:
+            self.worst_status_current = 'UNREACHABLE'
+        elif self.disaster > 0:
+            self.worst_status_current = 'DISASTER'
+        elif self.critical > 0:
+            self.worst_status_current = 'CRITICAL'
+        elif self.high > 0:
+            self.worst_status_current = 'HIGH'
+        elif self.average > 0:
+            self.worst_status_current = 'AVERAGE'
+        elif self.warning > 0:
+            self.worst_status_current = 'WARNING'
+        elif self.information > 0:
+            self.worst_status_current = 'INFORMATION'
+        elif self.unknown > 0:
+            self.worst_status_current = 'UNKNOWN'
+
+        return self.worst_status_current
+
+    def get_worst_status_diff(self):
+        """
+            hand over the current worst status difference for QUI
+        """
+        return self.worst_status_diff
